<?php

namespace Bixo\Leaderboard\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Bixo\Leaderboard\Http\Requests\KpiRequest;
use Bixo\Leaderboard\Interfaces\KpiRepositoryInterface;
use Bixo\Leaderboard\Models\Kpi;
use Bixo\Leaderboard\Forms\Kpi as Form;

/**
 * APIController  class for kpi.
 */
class KpiAPIController extends BaseController
{

    /**
     * Initialize kpi resource controller.
     *
     * @param type KpiRepositoryInterface $kpi
     *
     * @return null
     */
    public function __construct(KpiRepositoryInterface $kpi)
    {
        parent::__construct();
        $this->repository = $kpi;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Leaderboard\Repositories\Criteria\KpiResourceCriteria::class);
    }

    /**
     * Display a list of kpi.
     *
     * @return Response
     */
    public function index(KpiRequest $request)
    {
        return $this->repository
            ->setPresenter(\Bixo\Leaderboard\Repositories\Presenter\KpiPresenter::class)
            ->paginate();
    }

    /**
     * Display kpi.
     *
     * @param Request $request
     * @param Model   $kpi
     *
     * @return Response
     */
    public function show(KpiRequest $request, Kpi $kpi)
    {
        return $kpi->setPresenter(\Bixo\Leaderboard\Repositories\Presenter\KpiListPresenter::class);
        ;
    }

    /**
     * Create new kpi.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(KpiRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('leaderboard::kpi.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('leaderboard/kpi/' . $kpi->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('leaderboard/kpi');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the kpi.
     *
     * @param Request $request
     * @param Model   $kpi
     *
     * @return Response
     */
    public function update(KpiRequest $request, Kpi $kpi)
    {
        try {
            $data = $request->all();

            $kpi->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('leaderboard::kpi.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('leaderboard/kpi/' . $kpi->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('leaderboard/kpi/' . $kpi->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the kpi.
     *
     * @param Model   $kpi
     *
     * @return Response
     */
    public function destroy(KpiRequest $request, Kpi $kpi)
    {
        try {
            $kpi->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('leaderboard::kpi.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('leaderboard/kpi/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('leaderboard/kpi/' . $kpi->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
