<?php

namespace Bixo\Leaderboard\Policies;

use Litepie\User\Contracts\UserPolicy;
use Bixo\Leaderboard\Models\Kpi;

class KpiPolicy
{

    /**
     * Determine if the given user can view the kpi.
     *
     * @param UserPolicy $user
     * @param Kpi $kpi
     *
     * @return bool
     */
    public function view(UserPolicy $user, Kpi $kpi)
    {
        if ($user->canDo('leaderboard.kpi.view') && $user->isAdmin()) {
            return true;
        }

        return $kpi->user_id == user_id() && $kpi->user_type == user_type();
    }

    /**
     * Determine if the given user can create a kpi.
     *
     * @param UserPolicy $user
     * @param Kpi $kpi
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('leaderboard.kpi.create');
    }

    /**
     * Determine if the given user can update the given kpi.
     *
     * @param UserPolicy $user
     * @param Kpi $kpi
     *
     * @return bool
     */
    public function update(UserPolicy $user, Kpi $kpi)
    {
        if ($user->canDo('leaderboard.kpi.edit') && $user->isAdmin()) {
            return true;
        }

        return $kpi->user_id == user_id() && $kpi->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given kpi.
     *
     * @param UserPolicy $user
     * @param Kpi $kpi
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Kpi $kpi)
    {
        return $kpi->user_id == user_id() && $kpi->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given kpi.
     *
     * @param UserPolicy $user
     * @param Kpi $kpi
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Kpi $kpi)
    {
        if ($user->canDo('leaderboard.kpi.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given kpi.
     *
     * @param UserPolicy $user
     * @param Kpi $kpi
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Kpi $kpi)
    {
        if ($user->canDo('leaderboard.kpi.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
