<?php

namespace Bixo\Leaderboard\Providers;

use Illuminate\Support\ServiceProvider;

class LeaderboardServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'leaderboard');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'leaderboard');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerLeaderboard();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerLeaderboard()
    {
        $this->app->bind('leaderboard', function($app) {
            return new Leaderboard($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Leaderboard', 'Lavalite\Leaderboard\Facades\Leaderboard');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('bixo.leaderboard', function ($app) {
            return $this->app->make('Bixo\Leaderboard\Leaderboard');
        });

                // Bind Kpi to repository
        $this->app->bind(
            'Bixo\Leaderboard\Interfaces\KpiRepositoryInterface',
            \Bixo\Leaderboard\Repositories\Eloquent\KpiRepository::class
        );

        $this->app->register(\Bixo\Leaderboard\Providers\AuthServiceProvider::class);
                $this->app->register(\Bixo\Leaderboard\Providers\EventServiceProvider::class);
        
        $this->app->register(\Bixo\Leaderboard\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and leaderboard's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'bixo.leaderboard'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeLeaderboard::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['bixo.leaderboard'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('bixo/leaderboard.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/leaderboard')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/leaderboard')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
